

# from ...resources import Resource
from .. import SearchResource


class CompanySearchResult(SearchResource):
    attribute_names = [
        'locale'
        'name'
        'name_formatted'
        'link'
    ]

    def __init__(self, client, company_number=None, **kwargs):
        super(CompanySearchResult, self).__init__(client, id=company_number, **kwargs)



from __future__ import unicode_literals

import six

from .. import Resource, SearchableResourceMeta


class Company(Resource):
    path = 'company'
    search_path = 'search'
    attribute_names = [
        'duedil_url',
        # string the url of the full company profile on duedil.com
        'company_number',
        # string the company number
        'name',
        # string the company name
        'name_formated',
        # string a more readable version of the company name
        'registered_address',
        # obj Holds address information about the company
        'category',
        # string The category of company eg "Public Limited Company"
        'status',
        # string a string describing the status of company eg "In
        # Liquidation"
        'locale',
        # string Either "United Kingdom" or "Republic of Ireland"
        'previous_names',
        # array a collection containing one or more previous name
        # objects
        'sic_codes',
        # array a collection containing one or more SIC code objects
        'incorporation_date',
        # string when the company was incorporated. [YYYY-MM-DD]
        'accounts',
        # obj Information about the most recent accounts
        'returns',
        # obj information about the company's returns
    ]

    def __init__(self, api_key, company_number=None, **kwargs):
        super(Company, self).__init__(api_key=api_key, id=company_number, **kwargs)



from __future__ import unicode_literals

from .... import ProResource


class AccountDetailsFinancial(ProResource):
    full_endpoint = True
    attribute_names = [
        'id',
        # string Accounts ID
        'last_update',
        # dateTime Date of last update
        'date',
        # dateTime Accounting to date
        'type',
        # string Accounts type
        'account_status',
        # integer Accounts status
        'accountant_fees',
        # integer Accountant fees
        'acquisitions_and_disposals',
        # integer Acquisitions and disposals
        'accruals_deferred_income',
        # integer Accruals & deferred income
        'amortisation_of_tangibles',
        # integer Amortisation of tangibles
        'assets_available_for_sale_financial',
        # integer Financial assets available for sale
        'assets_deferred_tax',
        # integer Deferred tax assets
        'assets_financial',
        # integer Financial assets
        'assets_financial_due_after',
        # integer Financial assets due after 12 months
        'assets_financial_due_width',
        # integer Financial assets due within 12 months
        'assets_other_due_after',
        # integer Other assets due after 12 months
        'assets_trading',
        # integer Trading assets
        'assets_intangible',
        # integer Intangible assets
        'assets_investment',
        # integer Investment assets
        'assets_misc_current',
        # integer Miscellaneous current assets
        'assets_other',
        # integer Other assets
        'assets_other_due_within',
        # integer Other assets due within 1 year
        'assets_other_intangible',
        # integer Other intangible assets
        'bank_loan',
        # integer Bank loan
        'bank_overdraft',
        # integer Bank overdraft
        'capital_expenditure',
        # integer Capital expenditure
        'commercial_assets_tangible',
        # integer Tangible commercial assets
        'company',
        # integer Company registration numbers
        'creditors',
        # integer Creditors
        'customer_accounts_due_after',
        # integer Customer accounts due after 12 months
        'financial_assets_tangible',
        # integer Tangible financial assets
        'assets_total_current',
        # integer Total current assets
        'assets_total_fixed',
        # integer Total fixed assets
        'auditor_fees',
        # integer Auditor fees
        'cash',
        # integer Cash
        'cash_at_central_banks',
        # integer Cash at central banks
        'cash_year_start',
        # integer Cash year start
        'change_in_cash',
        # integer Change in cash
        'consolidated_accounts',
        # boolean Accounts consolidated (Y/N)
        'currency',
        # string Accounts currency
        'customer_accounts',
        # integer Customer accounts
        'customer_accounts_due_within',
        # integer Customer accounts due within 1 year
        'debt_securities',
        # integer Debt securities
        'debt_securities_due_after',
        # integer Debt securities due after one year
        'debt_securities_due_within',
        # integer Debt securities due within one year
        'debt_securities_in_issue',
        # integer Debt securities in issue
        'debt_securities_in_issue_due_after',
        # integer Debt securities in issue due after 12 months
        'debt_securities_in_issue_due_within',
        # integer Debt securities in issue due within 12 months
        'debtors',
        # integer Debtors
        'debtors_due_after',
        # integer Debtors due after 12 months
        'deposits_by_banks',
        # integer Deposits by banks
        'deposits_by_banks_due_after',
        # integer Deposits by banks due after 1 year
        'deposits_by_banks_due_within',
        # integer Deposits by banks due within 1 year
        'depreciation_of_tangibles',
        # integer Depreciation of tangibles
        'derivative_financial_instruments',
        # integer Derivative financial instruments
        'derivatives',
        # integer Derivatives
        'derivatives_due_after',
        # integer Derivatives due after 12 months
        'derivatives_due_within',
        # integer Derivatives due within 12 months
        'director_other',
        # integer Director other
        'director_pensions',
        # integer Director pensions
        'director_social_security',
        # integer Director social security
        'directors_accounts',
        # integer Directors accounts
        'director_fees',
        # integer Director fees
        'directors_remuneration',
        # integer Directors' remuneration
        'dividends',
        # integer Dividends
        'dividends_other',
        # integer Other dividends
        'employee_costs',
        # integer Employee costs
        'employee_numbers',
        # integer Employee numbers
        'employee_other',
        # integer Employee other
        'employee_pensions',
        # integer Employee pensions
        'employee_remuneration',
        # integer Employee remuneration
        'employee_social_security',
        # integer Employee social security
        'equity_dividends_paid',
        # integer Equity dividends paid
        'equity_shares',
        # integer Equity shares
        'exceptional_items',
        # integer Exceptional items
        'exceptional_other_items',
        # integer Exceptional other items
        'exceptional_pandl_on_disposal',
        # integer Exceptional profit & loss on disposal
        'exceptional_pandl_on_reorganisations',
        # integer Exceptional profit & loss on reorganisations
        'exchange_rate_effect',
        # integer Exchange rate effect
        'fees_and_commission_expense',
        # integer Fees & commission expense
        'fees_and_commission_income',
        # integer Fees & commission income
        'financing_activities',
        # integer Financing activities
        'goodwill',
        # integer Goodwill
        'group_accounts',
        # integer Group accounts
        'group_debtors',
        # integer Group debtors
        'highest_paid_director',
        # integer Highest paid director
        'hp_commitments',
        # integer HP commitments
        'interest_and_similar_expense',
        # integer Interest & similar expense
        'interest_and_similar_income',
        # integer Interest & similar income
        'investing_activities',
        # integer Investing activities
        'investment_and_other',
        # integer Investment & other
        'investment_property',
        # integer Investment property
        'items_in_course_of_collection',
        # integer Items in course of collection
        'items_in_course_of_transmission',
        # integer Items in course of transmission
        'lease_commitments',
        # integer Lease commitments
        'liabilities_current_tax',
        # integer Current tax liabilities
        'liabilities_deferred_tax',
        # integer Deferred tax liabilities
        'liabilities_financial',
        # integer Financial liablities
        'liabilities_financial_due_after',
        # integer Financial liabilities due after 12 months
        'liabilities_financial_due_within',
        # integer Financial liabilities due within 12 months
        'liabilities_insurance',
        # integer Insurance liabilities
        'liabilities_other_due_after',
        # integer Other liabilities due after one year
        'liabilities_other_provisions',
        # integer Liabilities - other provisions
        'liabilities_subordinated_due_after',
        # integer Subordinated liabilities due after one year
        'liabilities_trading',
        # integer Trading liabilities
        'liabilities_other',
        # integer Other liabilities
        'liabilities_other_due_within',
        # integer Other Liabilities due within 1 year
        'liabilities_subordinated',
        # integer Subordinated liabilities
        'liabilities_subordinated_due_within',
        # integer Subordinated Liabilities due within 1 year
        'loans_and_advances_to_banks',
        # integer Loans & advances to banks
        'loans_and_advances_to_banks_due_after',
        # integer Loans & advances to banks due after 1 year
        'loans_and_advances_to_banks_due_within',
        # integer Loans & advances to banks due within 1 year
        'loans_and_advances_to_customers',
        # integer Loans & advances to customers
        'loans_and_advances_to_customers_due_after',
        # integer Loans & advances to customers due after 1 year
        'loans_and_advances_to_customers_due_within',
        # integer Loans & advances to customers due within 1 year
        'lt_bank_loans',
        # integer Long term bank loans
        'lt_directors_accounts',
        # integer Long term directors accounts
        'lt_group_accounts',
        # integer Long term group accounts
        'lt_hp_commitments',
        # integer Long term HP commitments
        'lt_lease_commitments',
        # integer Long term lease commitments
        'lt_loans',
        # integer Long term loans
        'lt_other_loans_finance',
        # integer Other long term finance loans
        'lt_total_accruals_deferred_income',
        # integer Long term total accruals deferred income
        'lt_total_hp_lease_commitments',
        # integer Long term total HP lease commitments
        'lt_total_liabilities',
        # integer Long term total liabilities
        'management_of_liquid_resources',
        # integer Management of liquid resources
        'minority_interests',
        # integer Minority interests
        'minority_interests_profit',
        # integer Minority interests profit
        'misc_debtors',
        # integer Miscellaneous debtors
        'misc_liabilities',
        # integer Miscellaneous liabilities
        'months',
        # integer Months included in accounts
        'net_cashflow_from_financing',
        # integer Net cashflow from financing
        'net_change_in_cash',
        # integer Net change in cash
        'net_fees_and_commission_income',
        # integer Net fees & commission income
        'net_interest_income',
        # integer Net interest income
        'net_pension_liability',
        # integer Net pension liability
        'net_tax_paid',
        # integer Net tax paid
        'net_trading_income',
        # integer Net trading income
        'operating_activities',
        # integer Operating activities
        'operating_expenses',
        # integer Operating expenses
        'operating_profit',
        # integer Operating profit
        'ordinary_shares',
        # integer Ordinary shares
        'other_audit_costs',
        # integer Other audit costs
        'other_income',
        # integer Other income
        'other_reserves',
        # integer Other reserves
        'other_appropriations',
        # integer Other appropriations
        'other_current_liability',
        # integer Other current liability
        'other_lt_liabilities',
        # integer Other long term liabilities
        'other_provisions_for_liabilities',
        # integer Other provisions for liabilities
        'other_shares',
        # integer Other shares
        'other_st_loans',
        # integer Other short term loans
        'pandl_revenue_reserve',
        # integer P&L revenue reserve
        'pre_tax_profit',
        # integer Pre-tax profit
        'preference_shares',
        # integer Preference shares
        'prepayments_accrued_income',
        # integer Prepayments & accrued income
        'profit_after_tax',
        # integer Profit after tax
        'retained_profit',
        # integer Retained profit
        'return_on_investments',
        # integer Return on investments
        'revaluation_reserve',
        # integer Revaluation reserve
        'share_premium_account',
        # integer Share premium account
        'share_profit_in_ventures',
        # integer Share profit in ventures
        'statutory_audit_costs',
        # integer Statutory audit costs
        'stocks_work_in_progress',
        # integer Stocks work in progress
        'tax',
        # integer Tax
        'total_current_liabilities',
        # integer Total current liabilities
        'total_current_liabilities',
        # integer Total long term liabilities
        'total_operating_income',
        # integer Total operating income
        'total_shareholders_funds',
        # integer Total shareholders' funds
        'total_called_issued_capital',
        # integer Total called issued capital
        'total_lt_liabilities',
        # integer Total long term liabilities
        'total_other_creditors',
        # integer Total other creditors
        'total_provisions',
        # integer Total provisions
        'trade_creditors',
        # integer Trade creditors
        'trade_debtors',
        # integer Trade debtors
        'treasury_other_bills',
        # integer Other treasury bills
        'year_end_cash_equivalents',
        # integer Year end cash equivalents
    ]




from importlib import import_module
from collections import Sequence



class SearchResource(object):
    attribute_names = None
    locale = 'uk'
    id = None
    path = None
    result_obj = {}

    def __init__(self, client, id=None, locale='uk', load=False, **kwargs):
        if not self.attribute_names:
            raise NotImplementedError(
                "Resources must include a list of allowed attributes")

        self.id = id
        assert(locale in ['uk', 'roi'])
        self.locale = locale
        self.client = client
        self.should_load = True if load else False
        if load:
            self.load()

        if kwargs:
            self._set_attributes(**kwargs)

    def _set_attributes(self, missing=False, **kwargs):
        for k, v in kwargs.items():
            if k in self.attribute_names:
                self.__setattr__(k, v)

        if missing is True:
            for allowed in self.attribute_names:
                if allowed not in kwargs:
                    self.__setattr__(allowed, None)

    def load(self):
        result = self.client.get(self.endpoint)
        self._set_attributes(**result)

    def __getattr__(self, name):
        """
        lazily return attributes, only contact duedil if necessary
        """
        try:
            return super(SearchResource, self).__getattribute__(name)
        except AttributeError:
            if name in self.attribute_names:
                self.load()
                return super(SearchResource, self).__getattribute__(name)
            elif name in self.result_obj.keys():
                # Assumes subclass has dict defined called result_obj, this maps attribute name to
                # string of module path including class to be instaniated.
                # The below dynamically imports the module and then gets the class, then returns and instance of the class
                mod_path, klass_str = self.result_obj[name].rsplit('.', 1)
                mod = import_module(mod_path)
                klass = getattr(mod, klass_str)
                return klass(self.id, client=self.client, locale=self.locale, load=self.should_load)
            else:
                raise

    def __eq__(self, other):
        if hasattr(other, 'id'):
            return self.id == other.id
        return self.id == other

    def __str__(self):
        return 'Search Result: {} ({})'.format(self.name, self.id)



class SearchResouceList(Sequence):

    def __init__(self, results, result_klass, client):
        self._result_list = []
        self.result_klass = result_klass
        self.client = client
        self.result_list = results

    def __len__(self):
        return len(self.result_list)

    def __getitem__(self, key):
        return self.result_list[key]

    def __iter__(self):
        return iter(self.result_list)

    def __contains__(self, result):
        return result in self.result_list

    def __eq__(self, other):
        rlist = self.result_list == other.result_list
        client = self.client == other.client
        rclass = self.result_klass == other.result_klass
        return rlist and client and rclass

    def __add__(self, other):
        if self.client.api_key == other.client.api_key:
            return self.result_list + other.result_list
        raise TypeError('Cannot join results from 2 different applications (api keys)')

    def __radd__(self, other):
        return self.__add__(other)

    @property
    def result_list(self):
        return self._result_list

    @result_list.setter
    def result_list(self, value):
        temp = [self.result_klass(self.client, **r) for r in value.get('response',{}).get('data', {})]
        if not self._result_list:
            self._result_list = temp
        else:
            self._result_list.extend(temp)



# -*- coding: utf-8 -*-
#
#  DuedilApiClient
#  @copyright 2014 Christian Ledermann
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
#

"""
Python `Duedil API <https://developer.duedil.com/>`_ Wrapper, written to access the v3 API.

Lets you interact easily with Duedil directly from your Python code. The project is
hosted on `Github <https://github.com/founders4schools/duedilv3>`_ and is referenced on
`PyPI <https://pypi.python.org/pypi/duedil>`_.

Installation
------------

As all PyPI hosted package, installation is very simple, just run::

    sudo pip install duedil

"""



from __future__ import unicode_literals

from ... import ProResource


class Directorship(ProResource):
    attribute_names = [
        'id',
        # string Director ID
        'last_update',
        # dateTime Date last updated
        'active',
        # boolean Active (true/false)
        'status',
        # string Status
        'founding',
        # boolean Founding director (true/false)
        'appointment_date',
        # dateTime Date appointed
        'function',
        # string Function
        'function_code',
        # integer Function code
        'position',
        # string Position
        'position_code',
        # string Position code
        'companies_url',
        # string Link to companies
        'directors_uri',
        # string Link to director profile
        'service_address_uri',
        # string Link to service address
        'address1',
        # string Address line 1
        'address2',
        # string Address line 2
        'address3',
        # string Address line 3
        'address4',
        # string Address line 4
        'address5',
        # string Address line 5
        'postal_area',
        # string Postal area
        'postcode',
        # string Postcode
        'owning_company',
        'resignation_date',
        'secretary',
    ]



from __future__ import unicode_literals

from .... import ProResource


class AccountDetailsGAAP(ProResource):
    full_endpoint = True
    attribute_names = [
        'id',
        # string Accounts ID
        'company',
        # string Company registration number
        'last_update',
        # dateTime Date of last update
        'date',
        # dateTime Accounting to date
        'type',
        # string Type of accounts
        'accountant_fees',
        # string Accountant fees
        'account_status',
        # integer Account status
        'accruals_deferred_income',
        # integer Accruals & deferred income
        'accruals_deferred_income_due',
        # integer Accruals & deferred income due after 1 year
        'acquisitions_and_disposals',
        # string Acquisitions and disposals
        'amortisation_of_intangibles',
        # integer Amortisation of intangibles
        'assets_intangible',
        # integer Intangible assets
        'assets_misc_current',
        # string Current miscellaneous
        'assets_tangible',
        # integer Tangible assets
        'assets_total_current',
        # integer Total current assets
        'assets_total_fixed',
        # integer Total fixed assets
        'auditor_fees',
        # integer Auditor fees
        'bank_interest_payable',
        # integer Bank interest payable
        'bank_interest_receivable',
        # string Bank interest receivable
        'bank_loans',
        # string Bank loans
        'bank_loans_lt',
        # string Long terms bank loans
        'bank_overdraft',
        # string Bank overdraft
        'capital_expenditure',
        # string Capital expenditure
        'cash',
        # integer Cash
        'cash_increase_decrease',
        # string Decrease/increase in cash
        'consolidated',
        # boolean Accounts consolidated (Y/N)
        'cost_of_sales',
        # string Cost of sales
        'creditors',
        # integer Creditors
        'currency',
        # string Accounts currency
        'current_hp_commitments',
        # string Current HP agreements
        'current_lease_commitments',
        # string Current lease commitments
        'debtors',
        # integer Debtors
        'debtors_due',
        # integer Debtors due
        'depreciation_of_tangibles',
        # integer Depreciation of tangibles
        'directors_accounts',
        # string Directors accounts
        'directors_accounts_lt',
        # string Directors long term accounts
        'director_other',
        # string Directors other
        'director_fees',
        # integer Director fees
        'director_pensions',
        # integer Director pensions
        'director_social_security',
        # string Director social security
        'directors_remuneration',
        # integer Directors' remenuration
        'dividends',
        # string Dividends
        'dividends_other',
        # string Dividends other
        'employee_costs',
        # integer Employee costs
        'employee_numbers',
        # integer Employee numbers
        'employee_pensions',
        # integer Employee pensions
        'employee_remuneration',
        # integer Employee remuneration
        'employee_social_security',
        # integer Employee social security
        'employee_other',
        # string Employee other
        'equity_dividends_paid',
        # string Equity dividends paid
        'exceptional_items',
        # string Exceptional items
        'exceptional_other_items',
        # string Exceptional other items
        'exceptional_pandl_on_disposal',
        # string Exceptional profit & loss on disposal
        'exceptional_pandl_on_reorganisations',
        # string Exceptional profit & loss on reorganisations
        'exports',
        # string Exports
        'financing_net_cashflow',
        # string Financing net cashflow
        'group_accounts_lt',
        # integer Group accounts
        'gross_profit',
        # string Gross profit
        'group_accounts',
        # string Group accounts
        'group_debtors',
        # string Group debtors
        'group_interest_receivable',
        # string Group interest receivable
        'highest_paid_director',
        # integer Highest paid director
        'hp_interest_payable',
        # string HP interest payable
        'interest_payable',
        # integer Interest payable
        'interest_receivable',
        # integer Interest receivable
        'investment_and_other',
        # integer Interest and other
        'lease_interest_payable',
        # string Lease interest payable
        'liabilities_deferred_tax',
        # string Deferred tax liabilities
        'liabilities_current_tax',
        # integer Current tax liabilities
        'liabilities_other_current',
        # integer Other current liabilities
        'liabilities_other_lt',
        # integer Other long term liabilities
        'liabilities_total_current',
        # integer Total current liabilities
        'liabilities_total_lt',
        # integer Total long term liabilities
        'liabilities_total_other_current',
        # integer Total other current liabilities
        'liabilities_total_other_non_current',
        # integer Total other non current liabilities
        'management_of_liquid_resources',
        # string Management of liquid resources
        'minority_interests',
        # string Minority interests
        'minority_interests_profit',
        # string Minority interests profit
        'misc_debtors',
        # integer Miscellaneous debtors
        'months',
        # integer Months included in accounts
        'net_operations_cashflow',
        # string Net operations cashflow
        'net_pension_liability',
        # string Net pension liability
        'non_current_hp_commitments',
        # string Non current HP commitments
        'non_current_lease_commitments',
        # string Non current lease commitments
        'operating_profit',
        # integer Operating profit
        'ordinary_shares',
        # integer Ordinary shares
        'other_audit_costs',
        # integer Other audit costs
        'other_interest_payable',
        # integer Other interest payable
        'other_loans_lt',
        # integer Other long term loans
        'other_provisions',
        # integer Other provisions
        'other_reserves',
        # integer Other reserves
        'other_appropriations',
        # string Other appropriations
        'other_interest_receivable',
        # string Other interest receivable
        'other_shares',
        # string Other shares
        'other_st_loans',
        # string Other short term loans
        'pandl_revenue_reserve',
        # integer P&L revenue reserve
        'pre_tax_profit',
        # integer Pre-tax profit
        'preference_shares',
        # string Preference shares
        'profit_after_tax',
        # integer Profit after tax
        'retained_profit',
        # integer Retained profit
        'return_on_investment',
        # string Return on investment
        'revaluation_reserve',
        # string Revaluation reserve
        'statutory_audit_costs',
        # integer Statutory audit costs
        'stocks_and_work_in_progress',
        # integer Stocks and work in progress
        'tax',
        # integer Tax
        'taxation',
        # string Taxation
        'total_called_issued_capital',
        # integer Total called issued capital
        'total_lt_loans',
        # integer Total long term loans
        'total_non_current_hp_lease_commitments',
        # string Total non-current HP lease commitments
        'total_provisions',
        # integer Total provisions
        'total_shareholders_funds',
        # integer Total shareholders' funds
        'trade_creditors',
        # integer Trade creditors
        'trade_debtors',
        # integer Trade debtors
        'turnover',
        # integer Turnover
    ]



from __future__ import unicode_literals

from ... import ProResource


class Document(ProResource):
    attribute_names = [
        'id',
        # string Document ID
        'last_update',
        # dateTime Date of last update
        'documentCode',
        # string Document code
        'date',
        # string Date filed
        'description',
        # string Description of Document Filing
    ]



# -*- coding: utf-8 -*-
#
#  DuedilApiClient v3 Pro + Credit
#  @copyright 2014 Christian Ledermann
#  @copyright 2015 Andrew Miller
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
#
#

import hashlib
from dogpile.cache import make_region
import json

# from dogpile.cache.proxy import ProxyBackend
# from requests import Response
# from urlparse import urlsplit
#
# class HttpProxy(ProxyBackend):
#     def set(self, key, value):
#         # value should be a http response object
#         assert isinstance(value, Response)
#         value = value.json()
#         params = value.url
#         self.proxied.set(key, value)


def kwargs_key_generator(namespace, fn, **kw):
    fname = fn.__name__
    def generate_key(*args, **kwargs):
        args_str = "_".join(str(s) for s in args)
        kwargs_str = json.dumps(kwargs)
        key = '{}_{}:{}_{}'.format(namespace, fname, args_str, kwargs_str)
        hashkey = hashlib.md5(key.encode('utf-8'))
        return hashkey.hexdigest()
    return generate_key

dp_region = make_region(name='duedilv3', function_key_generator = kwargs_key_generator)


def configure_cache(backend='dogpile.cache.pylibmc', expiration_time=86400, **kwargs):
    if not kwargs:
        kwargs = {
            'url': ["127.0.0.1"],
        }
    return dp_region.configure(
        backend,
        expiration_time = expiration_time, # 1 day
        arguments = kwargs
    )



from __future__ import unicode_literals

from .... import ProResource


class AccountDetailsStatutory(ProResource):
    full_endpoint = True
    attribute_names = [
        'id',
        # string Accounts ID
        'last_update',
        # dateTime Date of last update
        'company',
        # string Company registration number
        'date',
        # string Accounting to date
        'type',
        # string Type of accounts
        'account_status',
        # integer Account status
        'accountants',
        # integer Accountants
        'accounts_format',
        # integer Accounts format
        'auditors',
        # string Auditor name
        'assets_current',
        # integer Current assets
        'assets_net',
        # integer Net assets
        'assets_other_current',
        # integer Other current assets
        'assets_tangible',
        # integer Tangible assets
        'assets_total',
        # integer Total assets
        'assets_total_current',
        # integer Total current assets
        'assets_total_fix',
        # integer Total fixed assets
        'assets_current_delta',
        # integer The change in the current assets value from the previous
        # year's filing to latest filing
        'assets_current_delta_percentage',
        # integer The percentage change in the current assets value from the
        # previous year's filing to latest filing
        'assets_intangible',
        # integer Intangible assets
        'assets_intangible_delta',
        # integer The change in the intangible assets value from the previous
        # year's filing to latest filing
        'assets_intangible_delta_percentage',
        # integer The percentage change in the current assets value from the
        # previous year's filing to latest filing
        'assets_net_delta',
        # integer The change in net assets value from the previous year's
        # filing to latest filing
        'assets_net_delta_percentage',
        # integer The percentage change in net assets value from the previous
        # year's filing to latest filing
        'assets_other_current_delta',
        # integer The change in other assets value from the previous year's
        # filing to latest filing
        'assets_other_current_delta_percentage',
        # integer The percentage change in other assets value from the previous
        # year's filing to latest filing
        'assets_tangible_delta',
        # integer The change in tangible assets value from the previous year's
        # filing to latest filing
        'assets_tangible_delta_percentage',
        # integer The percentage change in tangible assets value from the
        # previous year's filing to latest filing
        'assets_total_current_delta',
        # integer The change in total current assets value from the previous
        # year's filing to latest filing
        'assets_total_current_delta_percentage',
        # integer The percentage change in total current assets value from the
        # previous year's filing to latest filing
        'assets_total_delta',
        # integer The change in total current assets value from the previous
        # year's filing to latest filing
        'assets_total_delta_percentage',
        # integer The percentage change in total current assets value from the
        # previous year's filing to latest filing
        'assets_total_fix_delta',
        # integer The change in total fixed assets value from the previous
        # year's filing to latest filing
        'assets_total_fix_delta_percentage',
        # integer The change in total fixed assets value from the previous
        # year's filing to latest filing
        'audit_fees',
        # integer Audit fees
        'audit_fees_delta',
        # integer The change in audit fees from the previous year's filing
        # to latest filing
        'audit_fees_delta_percentage',
        # integer The percentage change in audit fees from the previous year's
        # filing to latest filing
        'bank_overdraft',
        # integer Bank overdraft
        'bank_overdraft_delta',
        # integer The change in bank overdraft from the previous year's filing
        # to latest filing
        'bank_overdraft_delta_percentage',
        # integer The percentage change in bank overdraft from the previous
        # year's filing to latest filing
        'bank_overdraft_lt_loans_delta',
        # integer The change in bank overdraft & long term loans from the
        # previous year's filing to latest filing
        'bank_overdraft_lt_loans_delta_percentage',
        # integer The percentage change in bank overdraft & long term loans
        # from the previous year's filing to latest filing
        'bank_overdraft_lt_loans',
        # integer Bank overdraft & long term loans
        'capital_employed',
        # integer Capital employed
        'capital_employed_delta',
        # integer The change in total capital employed from the previous year's
        # filing to latest filing
        'capital_employed_delta_percentage',
        # integer The percentage change in total capital employed value from
        # the previous year's filing to latest filing
        'cash',
        # integer Cash
        'cash_delta',
        # integer The change in total cash from the previous year's filing to
        # latest filing
        'cash_delta_percentage',
        # integer The percentage change in total cash value from the previous
        # year's filing to latest filing
        'company',
        # integer Company registration number
        'consolidated',
        # boolean Consolidated accounts (Y/N)
        'contingent_liability',
        # integer Contingent liability
        'contingent_liability_delta',
        # integer The change in contingent liability from the previous year's
        # filing to latest filing
        'contingent_liability_delta_percentage',
        # integer The percentage change in contingent liability from the
        # previous year's filing to latest filing
        'cost_of_sales',
        # integer Cost of sales
        'cost_of_sales_delta',
        # integer The change in cost of sales from the previous year's filing
        # to latest filing
        'cost_of_sales_delta_percentage',
        # integer The percentage change in cost of sales from the previous
        # year's filing to latest filing
        'currency',
        # string Accounts currency
        'debtor_days',
        # integer Debtor days
        'depreciation',
        # integer Depreciation
        'depreciation_delta',
        # integer The change in depreciation from the previous year's filing to
        # latest filing
        'depreciation_delta_percentage',
        # integer The percentage change in depreciation from the previous
        # year's filing to latest filing
        'directors_emoluments_delta',
        # integer The change in directors' emoluments from the previous year's
        # filing to latest filing
        'directors_emoluments_delta_percentage',
        # integer The percentage change in directors' emoluments from the
        # previous year's filing to latest filing
        'directors_emoluments',
        # integer Directors' emoluments
        'dividends_payable',
        # integer Dividends payable
        'dividends_payable_delta',
        # integer The change in dividends payable from the previous year's
        # filing to latest filing
        'dividends_payable_delta_percentage',
        # integer The percentage change in dividends payable from the previous
        # year's filing to latest filing
        'exports',
        # integer Exports
        'exports_delta',
        # integer The change in exports from the previous year's filing to
        # latest filing
        'exports_delta_percentage',
        # integer The percentage change in exports from the previous year's
        # filing to latest filing
        'gross_profit',
        # integer Gross profit
        'gross_profit_delta',
        # integer The change in gross profit from the previous year's filing to
        # latest filing
        'gross_profit_delta_percentage',
        # integer The percentage change in profit from the previous year's
        # filing to latest filing
        'increase_in_cash',
        # integer Increase in cash
        'increase_in_cash_delta',
        # integer The change in increase in cash from the previous year's
        # filing to latest filing
        'increase_in_cash_delta_percentage',
        # integer The percentage change in increase in cash from the previous
        # year's filing to latest filing
        'interest_payments',
        # integer Interest payments
        'interest_payments_delta',
        # integer The change in interest payments from the previous year's
        # filing to latest filing
        'interest_payments_delta_percentage',
        # integer The percentage change in interest payments from the previous
        # year's filing to latest filing
        'joint_auditors',
        # integer Joint auditors
        'liabilities_current',
        # integer Current liabilities
        'liabilities_lt',
        # integer Long term liabilities
        'liabilities_misc_current',
        # integer Miscellaneous current liabilities
        'liabilities_total',
        # integer Total liabilities
        'liabilities_current_delta',
        # integer The change in current liabilities from the previous year's
        # filing to latest filing
        'liabilities_current_delta_percentage',
        # integer The change in current liabilities from the previous year's
        # filing to latest filing
        'liabilities_lt_delta',
        # integer The change in long terms liablilties from the previous year's
        # filing to latest filing
        'liabilities_lt_delta_percentage',
        # integer The percentage change in long term liabilities from the
        # previous year's filing to latest filing
        'liabilities_misc_current_delta',
        # integer The change in miscellaneous current liabilities from the
        # previous year's filing to latest filing
        'liabilities_misc_current_delta_percentage',
        # integer The percentage change in miscellaneous current liabilities
        # from the previous year's filing to latest filing
        'liabilities_total_delta',
        # integer The change in total liabilities from the previous year's
        # filing to latest filing
        'liabilities_total_delta_percentage',
        # integer The percentage change in total liabilities from the previous
        # year's filing to latest filing
        'lt_loans',
        # integer Long term loans
        'lt_loans_delta',
        # integer The change in long term loans from the previous year's filing
        # to latest filing
        'lt_loans_delta_percentage',
        # integer The percentage change in long term loans from the previous
        # year's filing to latest filing
        'months',
        # integer Months included in accounts
        'net_cashflow_before_financing',
        # integer Net cashflow before financing
        'net_cashflow_from_financing',
        # integer Net cashflow from financing
        'net_cashflow_before_financing_delta',
        # integer The change in net cashflow before financing from the previous
        # year's filing to latest filing
        'net_cashflow_before_financing_delta_percentage',
        # integer The percentage change in net cashflow before financing from
        # the previous year's filing to latest filing
        'net_cashflow_from_financing_delta',
        # integer The change in net cashflow from financing from the previous
        # year's filing to latest filing
        'net_cashflow_from_financing_delta_percentage',
        # integer The percentage change in net cashflow from financing from the
        # previous year's filing to latest filing
        'net_worth_delta',
        # integer The change in net worth from the previous year's filing to
        # latest filing
        'net_worth_delta_percentage',
        # integer The change in net worth from the previous year's filing to
        # latest filing
        'net_worth',
        # integer Net worth
        'no_of_employees',
        # integer Number of employees
        'no_of_employees_delta',
        # integer The change in number of employees from the previous year's
        # filing to latest filing
        'no_of_employees_delta_percentage',
        # integer The percentage change in number of employees from the
        # previous year's filing to latest filing
        'operating_profits',
        # integer Operating profits
        'operating_profits_delta',
        # integer The change in operating profits from the previous year's
        # filing to latest filing
        'operating_profits_delta_percentage',
        # integer The percentage change in operating profits from the previous
        # year's filing to latest filing
        'operations_net_cashflow_delta',
        # integer The change in operations net cashflow from the previous
        # year's filing to latest filing
        'operations_net_cashflow_delta_percentage',
        # integer The percentage change operations net cashflow from the
        # previous year's filing to latest filing
        'operations_net_cashflow',
        # integer Net cashflow
        'paid_up_equity',
        # integer Paid-up equity
        'paid_up_equity_delta',
        # integer The change in paid-up equity from the previous year's filing
        # to latest filing
        'paid_up_equity_delta_percentage',
        # integer The percentage change in paid-up equity from the previous
        # year's filing to latest filing
        'pandl_account_reserve',
        # integer Profit & loss account reserve
        'pandl_account_reserve_delta',
        # integer The change in profit & loss account reserve from the previous
        # year's filing to latest filing
        'pandl_account_reserve_delta_percentage',
        # integer The percentage change in profit & loss account reserve from
        # the previous year's filing to latest filing
        'pre_tax_profit',
        # integer Pre-tax profit
        'pre_tax_profit_delta',
        # integer The change in pre-tax profit from the previous year's filing
        # to latest filing
        'pre_tax_profit_delta_percentage',
        # integer The percentage change in pre-tax profit from the previous
        # year's filing to latest filing
        'profit_after_tax',
        # integer Profit after tax
        'profit_after_tax_delta',
        # integer The change in profit after tax from the previous year's
        # filing to latest filing
        'profit_after_tax_delta_percentage',
        # integer The percentage change in profits after tax from the previous
        # year's filing to latest filing
        'qualification_code',
        # integer Qualification codes:
        #    0 = No Adverse Comments;
        #    1 = Exempt from audit;
        #    2 = The audit report contains no adverse comments;
        #    3 = The audit report contains additional comments;
        #    4 = The auditors opinions have been limited by the information
        #        provided to them;
        #    5 = The auditors have been unable to give a full opinion due to a
        #        disagreement over accounting practice;
        #    6 = The audit report states that there is uncertainty concerning
        #        the company being a going concern;
        #    7 = The audit report is qualified on more than one matter or there
        #        is an adverse opinion.
        'retained_profit',
        # integer Retained profit
        'retained_profit_delta',
        # integer The change in retained profit from the previous year's filing
        # to latest filing
        'retained_profit_delta_percentage',
        # integer The percentage change in retained from the previous year's
        # filing to latest filing
        'return_on_assets_ratio',
        # integer Return on assets ratio
        'revaluation_reserve',
        # integer Revaluation reserve
        'revaluation_reserve_delta',
        # integer The change in revaluation reserve from the previous year's
        # filing to latest filing
        'revaluation_reserve_delta_percentage',
        # integer The percentage change in revaluation reserve from the
        # previous year's filing to latest filing
        'shareholder_funds',
        # integer Shareholder funds
        'shareholder_funds_delta',
        # integer The change in shareholder funds from the previous year's
        # filing to latest filing
        'shareholder_funds_delta_percentage',
        # integer The change in shareholder funds from the previous year's
        # filing to latest filing
        'short_term_loans',
        # integer Short term loans
        'short_term_loans_delta',
        # integer The change in short term loans from the previous year's
        # filing to latest filing
        'short_term_loans_delta_percentage',
        # integer The percentage change in short term loans from the previous
        # year's filing to latest filing
        'stock',
        # integer Stock
        'solicitors',
        # integer Solicitors
        'stock_delta',
        # integer The change in stock from the previous year's filing to latest
        # filing
        'stock_delta_percentage',
        # integer The percentage change in stock from the previous year's
        # filing to latest filing
        'sundry_reserves',
        # integer Sundry reserves
        'sundry_reserves_delta',
        # integer The change in sundry reserves from the previous year's filing
        # to latest filing
        'sundry_reserves_delta_percentage',
        # integer The percentage change in sundry reserves from the previous
        # year's filing to latest filing
        'taxation',
        # integer Taxation
        'taxation_delta',
        # integer The change in taxation from the previous year's filing to
        # latest filing
        'taxation_delta_percentage',
        # integer The percentage change in taxation from the previous year's
        # filing to latest filing
        'trade_creditors',
        # integer Trade creditors
        'trade_creditors_delta',
        # integer The change in trade creditors from the previous year's filing
        # to latest filing
        'trade_creditors_delta_percentage',
        # integer The percentage change in trade creditors from the previous
        # year's filing to latest filing
        'turnover',
        # integer Turnover
        'trade_debtors',
        # integer Trade debtors
        'trade_debtors_delta',
        # integer The change in trade debtors from the previous year's filing
        # to latest filing
        'trade_debtors_delta_percentage',
        # integer The percentage change in trade debtors from the previous
        # year's filing to latest filing
        'turnover_delta',
        # integer The change in turnover from the previous year's filing to
        # latest filing
        'turnover_delta_percentage',
        # integer The percentage change in turnover from the previous year's
        # filing to latest filing
        'wages',
        # integer Wages
        'wages_delta',
        # integer The change in wages from the previous year's filing to latest
        # filing
        'wages_delta_percentage',
        # integer The percentage change in wages from the previous year's
        # filing to latest filing
        'working_capital',
        # integer Working capital
        'working_capital_delta',
        # integer The change in working capital from the previous year's filing
        # to latest filing
        'cash_to_current_liabilities_ratio',
        # float Cash to current liabilities ratio
        'cash_to_total_assets_ratio',
        # float Cash to total assets ratio
        'current_ratio',
        # float Current ratio
        'gearing',
        # float Gearing
        'gross_margin_ratio',
        # float Gross margin ratio
        'inventory_turnover_ratio',
        # float Inventory turnover ratio
        'liquidity_ratio',
        # float Liquidity ratio
        'net_profitability',
        # float Net profitability
        'profit_ratio',
        # float Profit ratio
        'retained_profits',
        # integer Retained profits
        'return_on_assets_ratio',
        # float Return on assets ratio
        'return_on_capital_employed',
        # float Return on capital employed
        'debt_to_capital_ratio',
        # float Debt to capital ratio
        'enterprise_value_to_revenue_multiple_ratio',
        # float Enterprise value to revenue multiple ratio
        'working_capital_delta_percentage',
        # integer The percentage change in working capital from the previous
        # year's filing to latest filing
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from ... import (Resource)


class Keywords(Resource):

    attribute_names = [
        'locale',
        'name',
        'keywords',
        'company_url',
    ]



from __future__ import unicode_literals

from ... import ProResource


class BankAccount(ProResource):
    attribute_names = [
        'bank',
        # string Name of bank
        'sortCode',
        # string Bank sort code
        'count',
        # integer Number of accounts
        'id',
        # string Identifier for bank account
    ]





# from ...resources import Resource
from .. import SearchResource


class CompanySearchResult(SearchResource):
    attribute_names = [
        'locale',
        'name',
        'company_url'
    ]
    result_obj = {
        'company': 'duedil.resources.pro.company.Company'
    }
    term_filters = [
        "locale",
        "location",
        "postcode",
        "sic_code",
        "sic_2007_code",
        "status",
        "currency",
        "keywords",
        "name",
    ]
    range_filters = [
        "employee_count",
        "turnover",
        "turnover_delta_percentage",
        "gross_profit",
        "gross_profit_delta_percentage",
        "cost_of_sales",
        "cost_of_sales_delta_percentage",
        "net_assets",
        "net_assets_delta_percentage",
        "current_assets",
        "current_assets_delta_percentage",
        "total_assets",
        "total_assets_delta_percentage",
        "cash",
        "cash_delta_percentage",
        "total_liabilities",
        "total_liabilities_delta_percentage",
        "net_worth",
        "net_worth_delta_percentage",
        "depreciation",
        "depreciation_delta_percentage",
        "taxation",
        "retained_profits",
        "profit_ratio",
        "inventory_turnover_ratio",
        "net_profitability",
        "return_on_capital_employed",
        "cash_to_total_assets_ratio",
        "gearing",
        "gross_margin_ratio",
        "return_on_assets_ratio",
        "current_ratio",
        "debt_to_capital_ratio",
        "cash_to_current_liabilities_ratio",
        "liquidity_ratio",
    ]



from __future__ import unicode_literals

from ... import ProResource


class Shareholder(ProResource):
    attribute_names = [
        'id',
        # string The registered ID of the company
        'last_update',
        # dateTime Date of last update
        'company',
        # string Company
        'title',
        # string Shareholders title
        'forename',
        # string Shareholders forename
        'surname',
        # string Shareholders surname
        'address1',
        # string Address 1
        'address2',
        # string Address 2
        'address3',
        # string Address 3
        'address4',
        # string Address 4
        'address5',
        # string Address 5
        'type',
        # string Type & class of the shares held
        'number',
        # integer The number of this type of share held
        'value',
        # integer The value of each share of this type held
        'currency',
        # string Currency
    ]



from __future__ import unicode_literals

from ... import ProResource


class ServiceAddress(ProResource):
    attribute_names = [
        'id',
        # string
        'last_update',
        # dateTime Date of last update
        'address1',
        # string Address part 1
        'address2',
        # string Address part 2
        'address3',
        # string Address part 3
        'address4',
        # string Address part 4
        'address5',
        # string Address part 5
        'postcode',
        # string Postcode
        'postal_area',
        # string Area code
    ]



from __future__ import unicode_literals

from .... import ProResource


class AccountDetailsIFRS(ProResource):
    full_endpoint = True
    attribute_names = [
        'id',
        # string Accounts ID
        'last_update',
        # dateTime Date of last update
        'date',
        # dateTime Accounting to date
        'type',
        # string Type of accounts
        'account_status',
        # integer Account status
        'accountant_fees',
        # integer Accountant fees
        'accruals_deferred_income',
        # integer Accruals & deferred income
        'accruals_deferred_income_due',
        # integer Accruals & deferred income due
        'amortisation_of_intangibles',
        # integer Amortisation of intangibles
        'assets_financial',
        # integer Financial assets
        'assets_financial_current',
        # integer Current financial assets
        'assets_intangible',
        # integer Intangible assets
        'assets_investment',
        # integer Investments
        'assets_other_current',
        # integer Other current assets
        'assets_other_non_current',
        # integer Other non-current assets
        'assets_tangible',
        # integer Tangible assets
        'assets_total_current',
        # integer Total current assets
        'assets_total_non_current',
        # integer Total non-current assets
        'auditor_fees',
        # integer Auditor fees
        'bank_interest_payable',
        # integer Interest payable
        'bank_interest_receivable',
        # integer Interest receivable
        'bank_loan',
        # integer Bank loan
        'bank_overdraft',
        # integer Bank overdraft
        'cash_equivalents',
        # integer Cash equivalents
        'cash_year_start',
        # integer Cash, year start
        'change_in_cash',
        # boolean Change in cash
        'company',
        # integer Company registration number
        'consolidated',
        # boolean Accounts consolidated (Y/N)
        'cost_of_sales',
        # integer Cost of sales
        'currency',
        # string Accounts currency
        'current_grants',
        # integer Current grant
        'current_hp_commitments',
        # integer Current HP lease commitments
        'current_lease_commitments',
        # integer Current lease commitments
        'debtors_due_after',
        # integer Debtors due after one year
        'depreciation_of_tangibles',
        # integer Depreciation of tangibles
        'director_fees',
        # integer Director fees
        'director_other',
        # integer Director other
        'director_pensions',
        # integer Director pensions
        'director_social_security',
        # integer Director social security
        'directors_accounts',
        # integer Directors' accounts
        'directors_remuneration',
        # integer Directors' remuneration
        'dividends_paid',
        # integer Dividends paid
        'employee_costs',
        # integer Employee costs
        'employee_numbers',
        # integer Employee numbers
        'employee_other',
        # integer Employee other
        'employee_pensions',
        # integer Employee pensions
        'employee_remuneration',
        # integer Employee remuneration
        'employee_social_security',
        # integer Employee social security
        'exceptional_items',
        # integer Exceptional items
        'exceptional_other_items',
        # integer Exceptional other items
        'exceptional_pandl_on_acquisition',
        # integer Exceptional profit & loss on acquisition
        'exceptional_pandl_on_reorganisations',
        # integer Exceptional profit & loss on reorganisations
        'exchange_rate_effect',
        # integer Exchange rate effect
        'exports',
        # integer Exports
        'financing_activities',
        # integer Financing activities
        'finished_goods',
        # integer Finished goods
        'gross_profit',
        # integer Gross profit
        'group_accounts',
        # integer Group accounts
        'group_accounts_payable',
        # integer Group accounts payable
        'group_debtors',
        # integer Group debtors
        'group_interest_receivable',
        # integer Group interest receivable
        'highest_paid_director',
        # integer Highest paid director
        'hp_interest_payable',
        # integer HP interest payable
        'interest_bearing_loans',
        # integer Interest bearing loans
        'interest_payable',
        # integer Interest payable
        'interest_receivable',
        # integer Interest receivable
        'inventories',
        # integer Inventories
        'investing_activities',
        # integer Investing activities
        'lease_interest_payable',
        # integer Lease interest payable
        'liabilities_current_resale',
        # integer Current resale liabilities
        'liabilities_non_current_resale',
        # integer Non-current resale liabilities
        'liabilities_other_current',
        # integer Other non-current liabilities
        'liabilities_current_financial',
        # integer Current financial liabilities
        'liabilities_current_tax',
        # integer Current tax liabilities
        'liabilities_deferred_tax',
        # integer Deferred tax liabilities
        'liabilities_non_current_financial',
        # integer Non current financial liabilities
        'liabilities_other_current_financial',
        # integer Other current financial liabilities
        'liabilities_other_non_current_financial',
        # integer Other non current financial liabilities
        'liabilities_pension',
        # integer Pension liabilities
        'liabilities_total_current',
        # integer Total current liabilities
        'liabilities_total_non_current',
        # integer Total non current liabilities
        'liabilities_total_other_current',
        # integer Total other current liabilities
        'liabilities_total_other_non_current',
        # integer Total other non current liabilities
        'minority_interests',
        # integer Minority interests
        'minority_interests_profit',
        # integer Minority interests profit
        'misc_debtors',
        # integer Miscellaneous debtors
        'months',
        # integer Month included in accounts
        'net_change_in_cash',
        # integer Net change in cash
        'non_current_lease_commitments',
        # integer Non current lease commitments
        'non_current_other_payables',
        # integer Non current other payables
        'operating_activities',
        # integer Operating activities
        'non_current_directors_loans',
        # integer Non-current director's loans
        'non_current_grants',
        # integer Non-current grants
        'non_current_group_accounts_payable',
        # integer Non-current group accounts payable
        'non_current_group_loans',
        # integer Non-current group loans
        'non_current_hp_commitments',
        # integer Non-current HP agreements
        'non_current_trade_payables',
        # integer Non-current trade payables
        'operating_profit',
        # integer Operating profit
        'operations_loss',
        # integer Operations loss
        'ordinary_shares',
        # integer Ordinary shares
        'other_audit_costs',
        # integer Other audit costs
        'other_payables',
        # integer Other payables
        'other_provisions',
        # integer Other provisions
        'other_receivables',
        # integer Other receivables
        'other_reserves',
        # integer Other reserves
        'other_appropriations',
        # integer Other appropriations
        'other_interest_payable',
        # integer Other interest payable
        'other_interest_receivable',
        # integer Other interest receivable
        'other_shares',
        # integer Other shares
        'pandl_revenue_reserve',
        # integer P&L revenue reserve
        'pre_tax_profit',
        # integer Pre-tax profit
        'preference_shares',
        # integer Preference shares
        'profit_after_tax',
        # integer Profit after tax
        'provisions',
        # integer Provisions
        'provisions_charges',
        # integer Provisions charges
        'raw_materials',
        # integer Raw materials
        'retained_profit',
        # integer Retained profit
        'revaluation_reserve',
        # integer Revaluation reserve
        'short_term_loans',
        # integer Short term loans
        'statutory_audit_costs',
        # integer Statutory audit costs
        'tax',
        # integer Tax
        'total_called_issued_capital',
        # integer Total called issued capital
        'total_shareholder_funds',
        # integer Total shareholder funds
        'trade_creditors',
        # integer Trade creditors
        'trade_debtors',
        # integer Trade debtors
        'trade_other_payables',
        # integer Trade other payables
        'turnover',
        # integer Turnover
        'work_in_progress',
        # integer Work in progress
        'year_end_cash_equivalents',
        # integer Year end cash equivalents
    ]



from __future__ import unicode_literals

from ... import ProResource


class Industry(ProResource):
    attribute_names = [
        'id',
        # string The registered ID of the company
        'sicCode',
        # string The additional SIC code the company has provided.
        'sicCodeDescription',
        # string Description of this SIC code.
    ]



'Accounts'
from __future__ import unicode_literals

from .... import ProResource, RelatedResourceMixin
import six
import sys


class Account(RelatedResourceMixin, ProResource):
    'Abstraction of Accounts resource in duedil v3 pro api'
    attribute_names = [
        'uri',
        'date',
        'type'
    ]
    account_classes = {
        'financial': 'pro.company.accounts.financial.AccountDetailsFinancial',
        'gaap': 'pro.company.accounts.gaap.AccountDetailsGAAP',
        'ifrs': 'pro.company.accounts.ifrs.AccountDetailsIFRS',
        'insurance': 'pro.company.accounts.insurance.AccountDetailsInsurance',
        'statutory': 'pro.company.accounts.statutory.AccountDetailsStatutory',
    }
    full_endpoint = True

    def __iter__(self):
        return iter(dict([(i,getattr(self, i)) for i in self.attribute_names]))

    @property
    def path(self):
        return self.uri.split('/', 5)[-1].rsplit('/', 1)[0]

    @property
    def details(self):
        resource = self.account_classes[self.type]

        if isinstance(resource, six.string_types):
            module, resource = resource.rsplit('.', 1)
            resource = getattr(sys.modules['duedil.resources.%s' % module], resource)
        resource_obj = self.load_related('details', resource, self.full_endpoint)
        resource_obj.path = '{}'.format(self.path)
        resource_obj.loaded = True
        return resource_obj



from __future__ import unicode_literals

from ... import ProResource


class PreviousCompanyName(ProResource):
    attribute_names = [
        'id',
        # string The registered ID of the company
        'last_update',
        # dateTime Date of last update
        'name',
        # string The previous company name
        'endedDate',
        # string The date this company stopped using this name
    ]



from .company import CompanySearchResult
from .director import DirectorSearchResult
from .. import SearchResouceList

from collections import Sequence
try:
    from urllib import urlencode
    import urlparse
except ImportError:
    from urllib.parse import urlencode
    from urllib import parse as urlparse

class ProSearchResourceList(SearchResouceList):

    def __init__(self, results, result_klass, client):
        super(ProSearchResourceList, self).__init__(results, result_klass, client)
        self._next_url = None
        # look at the property below
        self.next_url = results
        page = results.get('response', {}).get('pagination', {})
        self._length = page.get('total', len(self.result_list))

    def __str__(self):
        return "Pro Search Result List - total: {}".format(len(self))

    @property
    def next_url(self):
        return self._next_url

    @next_url.setter
    def next_url(self, value):
        page = value.get('response', {}).get('pagination', {})
        if page:
            self._next_url = page.get('next_url')
        else:
            self._next_url = ''


    def next(self):
        # should get the next set of results
        # update the internal list
        if not self.fetched_all_results():
            next_set = self.client.get(*self.parse_next_url())
            # check the property - this does a .extend()!
            self.result_list = next_set
            # update the next_url
            self.next_url = next_set
        else:
            raise StopIteration

    def __len__(self):
        return self._length

    def __getitem__(self, key):
        if type(key) is int:
            if key < 0:
                # key should be negative so +- equals -
                key = self._length + key
            if key >= self._length:
                raise IndexError()
            try:
                return self.result_list[key]
            except IndexError:
                # key is not in current cut of data
                # therefore loop with .next until result?
                self._update_next_url(1, key)
                return self.client.get(*self.parse_next_url())
        elif type(key) is slice:
            # this needs to be done to limit for loop iteration...
            raise NotImplementedError("Results don't support slicing at this time")
        else:
            raise TypeError()

    def __iter__(self):
        for result in self.result_list:
            yield result
            # get the last one and extend the list
            # if self.result_list[-1] is result:
            #     self.next()


    def __contains__(self, result):
        while not self.fetched_all_results():
            if result in self.result_list:
                return True
            # this could be hugely expensive, so get as many as possible. Duedil only allow upto a limit of 100
            self._update_next_url(limit=100)
            self.next()
        return False

    def fetched_all_results(self):
        return len(self.result_list) >= len(self)

    def _update_next_url(self, limit=None, offset=None):
        scheme, netloc, path, query_string, frag = urlparse.urlsplit(self._next_url)
        query_params = urlparse.parsed_qs(query_string)
        if limit:
            query_params['limit'] = [limit]
        if offset:
            query_params['offset'] = [offset]
        self._next_url = urlparse.urlunsplit((scheme, netloc, path, urlencode(query_params, doseq=True), frag))

    def parse_next_url(self):
        parsed_url = urlparse.urlsplit(self._next_url)
        path = parsed_url.path.rsplit('/', 1)[-1]
        if path.endswith('.json'):
            path = path[:-len('.json')] # grab the last part of the path
        query_params = dict(urlparse.parse_qsl(parsed_url.query))
        return path, query_params



from __future__ import unicode_literals

from ... import ProResource


class RegisteredAddress(ProResource):
    attribute_names = [
        # 'id',
        # string The registered ID of the company
        'last_update',
        # dateTime Date of last update
        'company',
        # string Company registration number
        'address1',
        # string Address part 1
        'address2',
        # string Address part 2
        'address3',
        # string Address part 3
        'address4',
        # string Address part 4
        'postcode',
        # string Postcode
        'phone',
        # string phone number
        'tps',
        # string TPS
        'website',
        # string Website
        'po_box',
        # string PO box number
        'care_of',
        # string Care of
        'email',
        # string Email address
        'area_code',
        # string Area code
    ]



from __future__ import unicode_literals

from .... import ProResource


class AccountDetailsInsurance(ProResource):
    full_endpoint = True
    attribute_names = [
        'account_status',
        # integer Account status
        'accountant_fees',
        # integer Accountant fees
        'accruals_and_deferred_income',
        # integer Accruals & deferred income
        'accrued_interest_and_rent',
        # integer Accrued interest and rent
        'acquisitions_and_disposals',
        # integer Acqisitions and discposals
        'amortisation_of_intangibles',
        # integer Amortisation of intangibles
        'amounts_owed_to_credit_institutions',
        # integer Debtors arising out of insurance operations
        'amounts_owed_to_group_undertakings',
        # integer Debtors arising out of group undertakings
        'assets_deferred_tax',
        # integer Deferred tax assets
        'assets_held_for_resale',
        # integer Assets held for resale
        'assets_intangible',
        # integer Intangible assets
        'assets_investment',
        # integer Investment asset
        'assets_other',
        # integer Other assets
        'assets_other_intangible',
        # integer Other intangible assets
        'assets_tangible',
        # integer Tangible assets
        'auditor_fees',
        # integer Auditor fees
        'borrowings',
        # integer Borrowings
        'borrowings_due_after',
        # integer Borrowings due after one year
        'borrowings_due_within',
        # integer Borrowings due within one year
        'capital_expenditure',
        # integer Capital expenditure
        'cash_and_bank_and_in_hand',
        # integer Cash at bank and in hand
        'cash_and_cash_equivalents',
        # integer Cash and cash equivalents
        'cash_year_start',
        # integer Cash at start of the year
        'change_in_cash',
        # integer Change in cash
        'company',
        # integer Company registration number
        'consolidated_accounts',
        # integer Consolidated accounts
        'creditors',
        # integer Creditors
        'currency',
        # integer Currency
        'current_tax_recoverable',
        # integer Current tax recoverable
        'debt_securities',
        # integer Debt securities
        'debtors',
        # integer Debtors
        'debtors_arising_out_of_group_undertakings',
        # integer Debtors arising out of group undertakings
        'debtors_arising_out_of_insurance_operations',
        # integer Debtors arising out of insurance operations
        'deferred_acquisition_costs',
        # integer Deferred acquisition costs
        'deposits_received_from_reinsurers',
        # integer Deposits received from reinsurers
        'depreciation_of_tangibles',
        # integer Depreciation of tangibles
        'direct_insurance_operations',
        # integer Direct insurance operations
        'director_fees',
        # integer Director fees
        'director_other',
        # integer Director other
        'director_pensions',
        # integer Director pensions
        'director_social_security',
        # integer Director social security
        'directors_remuneration',
        # integer Directors remuneration
        'discontinued_ops',
        # integer Discontinued ops
        'dividends',
        # integer Dividends
        'dividends_paid',
        # integer Dividends paid
        'employee_costs',
        # integer Employee costs
        'employee_numbers',
        # integer Employee numbers
        'employee_other',
        # integer Employee other
        'employee_pensions',
        # integer Employee pensions
        'employee_remuneration',
        # integer Employee remuneration
        'employee_social_security',
        # integer Employee
        'equity_dividends_paid',
        # integer Equity dividends paid
        'exceptional_items',
        # integer Exceptional items
        'exceptional_other_items',
        # integer Exceptional other items
        'exceptional_pandl_on_disposal',
        # integer Exceptional profit/loss on disposal
        'exceptional_pandl_on_reorganisations',
        # integer Exceptional profit/loss on reorganisations
        'exchange_rate_effect',
        # integer Exchange rate effect
        'fee_commission_income',
        # integer Fee & commission income
        'finance',
        # integer Finance
        'finance_costs',
        # integer Finance costs
        'financial_investments_after',
        # integer Financial investments after 12 months
        'financial_investments_within',
        # integer Financial investments within 12 months
        'financing_activities',
        # integer Financing activities
        'gaap_other_appropriations',
        # integer GAAP other appropriations
        'goodwill',
        # integer Goodwill
        'gross_written_premiums',
        # integer Gross written premiums
        'highest_paid_director',
        # integer Highest paid director
        'id',
        # integer ID
        'ifrs_other_appropriations',
        # integer IFRS other appropriations
        'insurance_debtors_and_assets',
        # integer Insurance debtors and assets
        'insurance_debtors_and_assets_settled_after',
        # integer Insurance debtors and assets to be settled after 12 months
        'insurance_debtors_and_assets_settled_within',
        # integer Insurance debtors and assets to be settled within 12 months
        'investing_activities',
        # integer Investing activities
        'investment',
        # integer Investment
        'investment_due_after',
        # integer Investment due after 12 months
        'investment_due_within',
        # integer Investments due within 12 months
        'investment_expenses_and_charges',
        # integer Investment expenses and charges
        'investment_income',
        # integer Investment income
        'investment_property',
        # integer Investment property
        'investments',
        # integer Investments
        'investments_in_group_undertakings',
        # integer Investments in group undertakings
        'land_and_buildings',
        # integer Land and buildings
        'less_income_tax_attributable_to_policyholders_returns',
        # integer Less income tax attributable to policyholders' returns
        'liabilities_current_tax',
        # integer Current tax liabilities
        'liabilities_deferred_tax',
        # integer Deferred tax liabilities
        'liabilities_held_for_sale',
        # integer Liabilities held for sale
        'liabilities_insurance',
        # integer Insurance liabilities
        'liabilities_insurance_settled_within',
        # integer Insurance liabilities to be settled within 12 months
        'liabilities_insurances_settled_after',
        # integer Insurance liabilities to be settled after 12 months
        'liabilities_other_non_insurance',
        # integer Other non-insurance liabilities
        'liabilities_reinsurance',
        # integer Reinsurance liabilities
        'liabilities_reinsurance_settled_after',
        # integer Reinsurance liabilities to be settled after 12 months
        'liabilities_reinsurance_settled_within',
        # integer Reinsurance liabilities to be settled within 12 months
        'loans_after',
        # integer Loans after 12 months
        'loans_and_deposits_with_credit_institutions',
        # integer Loans and deposits with credit institutions
        'loans_within',
        # integer Loans within 12 months
        'management_of_liquid_resources',
        # integer Management of liquid resources
        'minority_interests',
        # integer Minority interests
        'months',
        # integer Months
        'net_change_in_cash',
        # integer Net change in cash
        'net_change_in_provision_for_unearned_premiums',
        # integer Net change in provision for unearned premiums
        'net_operating_expense',
        # integer Net operating expense
        'net_pension_liability',
        # integer Net pensions liability
        'net_premiums_earned',
        # integer Net premiums earned
        'operating_activities',
        # integer Operating activities
        'ordinary_shares',
        # integer Ordinary shares
        'other_audit_costs',
        # integer Other audit costs
        'other_creditors',
        # integer Other creditors
        'other_debtors',
        # integer Other debtors
        'other_investments',
        # integer Other investments
        'other_investments_after',
        # integer Other investments after 12 months
        'other_investments_within',
        # integer Other investments within 12 months
        'other_liabilities',
        # integer Other liabilities
        'other_lt_loans',
        # integer Other long term loans
        'other_operating_income',
        # integer Other operating income
        'other_prepayments_and_accrued_income',
        # integer Other prepayments and accrued income
        'other_reserves',
        # integer Other reserves
        'other_shares',
        # integer Other shares
        'other_technical_income',
        # integer Other technical income
        'outward_reinsurance_premiums',
        # integer Outward reinsurance premiums
        'pandl_revenue_reserve',
        # integer Profit & loss / revenue reserve
        'plant_and_equipment',
        # integer Plant and equipment
        'pre_tax_profit',
        # integer Pre-tax profit
        'preference_shares',
        # integer Preference shares
        'prepayments_and_accrued_income',
        # integer Prepayments and accrued income
        'profit_after_tax',
        # integer Profit after tax
        'profit_attributable_to_minority_investments',
        # integer Profit attributable to minority interests
        'profit_before_tax_attributable_to_shareholders',
        # integer Profit before tax attributable to shareholders
        'profit_from_continuing_operations_after_tax',
        # integer Profit from continuing operations after tax
        'property',
        # integer Property
        'provisions',
        # integer Provisions
        'provisions_for_other_risks_and_charged',
        # integer Provisions for other risks and charges
        'reinsurance_debtors_and_assets',
        # integer Reinsurance debtors and assets
        'reinsurance_debtors_and_assets_settled_after',
        # integer Reinsurance debtors and assets settled after 12 months
        'reinsurance_debtors_and_assets_settled_within',
        # integer Reinsurance debtors and assets settled within 12 months
        'reinsurance_operations',
        # integer Reinsurance operations
        'reinsurers_share_of_technical_provisions',
        # integer Reinsurers share of technical provisions
        'retained_profit',
        # integer Retained profit
        'return_on_investment',
        # integer Return on investment
        'revaluation_reserve',
        # integer Revaluation reserve
        'share_premium',
        # integer Share premium
        'shares_and_other_variable_yield_securities',
        # integer Shares and other variable yield securities
        'statutory_audit_costs',
        # integer Statutory audit costs
        'subordinated_borrowings',
        # integer Subordinated borrowings
        'tax',
        # integer Tax
        'tax_attributable_to_policyholders_returns',
        # integer Tax attributable to policyholders' returns
        'tax_attributable_to_shareholders_profits',
        # integer Tax attributable to shareholders profits
        'tax_expense',
        # integer Tax expense
        'taxation',
        # integer Taxation
        'technical_provisions',
        # integer Technical provisions
        'total_called_issued_capital',
        # integer Total called up / issued capital
        'total_capital_reserves',
        # integer Total capital reserves
        'total_equity_and_reserves',
        # integer Total equity and reserves
        'total_shareholders_funds',
        # integer Total shareholders funds
        'turnover',
        # integer Turnover
        'type',
        # integer Type
        'year_end_cash_equivalents',
        # integer Year end cash equivalents
    ]



# -*- coding: utf-8 -*-
#
#  DuedilApiClient v3 Pro
#  @copyright 2014 Christian Ledermann
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
#

from __future__ import unicode_literals

import sys
import six
from collections import Mapping
from abc import ABCMeta

from ..api import LiteClient, ProClient  # , InternationalClient


class Resource(Mapping):
    attribute_names = None
    locale = 'uk'
    id = None
    path = None
    client_class = LiteClient
    loaded = False

    def __init__(self, id, api_key=None, locale='uk', load=False, client=None, **kwargs):
        if not self.attribute_names:
            raise NotImplementedError(
                "Resources must include a list of allowed attributes")

        self.id = id
        assert(locale in ['uk', 'roi'])
        self.locale = locale
        if client:
            self.client = client
        else:
            self.client = self.client_class(api_key, sandbox=kwargs.pop('sandbox', False))

        if load:
            self.load()

        if kwargs:
            self._set_attributes(**kwargs)

    def _set_attributes(self, missing=False, **kwargs):
        for k, v in kwargs.items():
            if k in self.attribute_names:
                self.__setattr__(k, v)

        if missing is True:
            for allowed in self.attribute_names:
                if allowed not in kwargs:
                    self.__setattr__(allowed, None)

    def load(self):
        self._result = self.client.get(self.endpoint)
        self.loaded = True
        self._set_attributes(**self._result)

    def __getattr__(self, name):
        """
        lazily return attributes, only contact duedil if necessary
        """
        try:
            return super(Resource, self).__getattribute__(name)
        except AttributeError:
            if name in self.attribute_names:
                if not self.loaded:
                    self.load()
                return super(Resource, self).__getattribute__(name)
            else:
                raise

    @property
    def endpoint(self):
        if not self.path:
            raise ValueError(
                "{model} does not have a path to load specified".format(
                    model=self.__class__.__name__))
        endpoint = '{locale}/{path}'.format(locale=self.locale,
                                            path=self.path)
        if self.id:
            endpoint += '/{id}'.format(id=self.id)
        return endpoint

    def __len__(self):
        return len(self.attribute_names)

    def __getitem__(self, key):
        return self.__getattr__(key)

    def __iter__(self):
        for prop in self.attribute_names + ['id']:
            if hasattr(self, prop):
                yield prop
            else:
                setattr(self, prop, None)
                yield prop

    def __contains__(self, key):
        if key in self._results.keys():
            return True
        if key in self._results.values():
            return True
        return False

    def __missing__(self, key):
        raise KeyError('%s in not a valid attribute' % key)

    def __str__(self):
        return '{} ({})'.format(self.__class__.__name__, self.id)

    def __eq__(self, other):
        return self.id == other.id


class ProResource(Resource):
    client_class = ProClient
    full_endpoint = False

    def load(self):
        """
        get results from duedil
        """
        self._result = self.client.get(self.endpoint)
        self.loaded = True
        self._set_attributes(**self._result['response'])


# Here be metaclass dragons that don't make complete sense as to why we have them


def resource_property(endpoint):
    def wrap(getter_fn):
        def inner(self, *args, **kwargs):
            return getter_fn(self, endpoint, *args, **kwargs)
        return inner
    return wrap


class SearchableResourceMeta(type):
    term_filters = None
    range_filters = None
    search_path = None

    def search(klass, order_by=None, limit=None, offset=None, **kwargs):
        data = klass.client._build_search_string(klass.term_filters,
                                                 klass.range_filters,
                                                 order_by=order_by, limit=limit,
                                                 offset=offset, **kwargs)
        results_raw = klass.client.get(klass.search_path, data=data)
        results = []
        for r in results_raw['response']['data']:
            results.append(
                klass(**r)
            )
        return results


class RelatedResourceMeta(ABCMeta):

    def __init__(klass, _name, _bases, ns):
        related_resources = ns.get('related_resources') or {}
        for ep in related_resources.keys():

            @resource_property(ep)
            def getter(self, endpoint):
                resource = self.related_resources[endpoint]

                if isinstance(resource, six.string_types):
                    module, resource = resource.rsplit('.', 1)
                    resource = getattr(sys.modules['duedil.resources.%s' % module], resource)

                return self.load_related(endpoint, resource, self.full_endpoint)

            attr_name = ep.replace('-', '_')
            setattr(klass, attr_name,
                    property(getter, None, None, attr_name))


class SearchableRelatedResourceMeta(SearchableResourceMeta, RelatedResourceMeta):
    pass


class RelatedResourceMixin(six.with_metaclass(RelatedResourceMeta, object)):
    related_resources = None

    def _get(self, resource, full_endpoint=False):
        if not full_endpoint:
            uri = '{endpoint}/{resource}'.format(endpoint=self.endpoint,
                                                 resource=resource)
        else:
            uri = self.endpoint
        return self.client.get(uri)

# need to deal with pagination...
    def load_related(self, key, klass=None, full_endpoint=False):
        internal_key = '_' + key.replace('-', '_')

        related = getattr(self, internal_key, None)

        if related is None:
            result = self._get(key, full_endpoint)
            if result:
                response = result['response']
                related = None
                if 'data' in response and isinstance(response['data'], (list, tuple)):
                    related = []
                    for r in result['response']['data']:
                        r['locale'] = r.get('locale', self.locale)
                        related.append(
                            klass(client=self.client, id=r.pop('id'), **r) if klass else None
                        )
                elif result:
                    locale = response.get('locale')
                    # HACK: for locale switching when traversing companies...
                    # XXX: this will need to be revisited!!!
                    if locale is None and 'company_url' in response:
                        path_components = response['company_url'][len(self.client.base_url):].split('/')
                        if 'uk' in path_components:
                            locale = 'uk'
                        elif 'roi' in path_components:
                            locale = 'roi'
                        else:
                            locale = 'uk'
                    else:
                        locale = self.locale
                    response['locale'] = locale
                    if klass:
                        related = klass(client=self.client, id=response.pop('id'), **response)
                setattr(self, internal_key, related)
        return related

    def __len__(self):
        return len(self.attribute_names + self.related_resources.keys())



from __future__ import unicode_literals


from ... import ProResource, RelatedResourceMixin


class Director(RelatedResourceMixin, ProResource):

    path = 'directors'

    attribute_names = [
        # 'id',
        # string Director ID
        'last_update',
        # dateTime Date last updated
        'open_directorships_count',
        # integer Number of open directorships
        'open_trading_directorships_count',
        # integer Number of open trading directorships
        'open_trading_director_directorships_count',
        # integer Of which a director
        'open_trading_secretary_directorships_count',
        # integer Of which a secretary
        'closed_directorships_count',
        # integer Number of closed directorships
        'retired_directorships_count',
        # integer Number of retired directorships
        'director_directorships_count',
        # integer Number of directorships (director)
        'open_director_directorships_count',
        # integer Number of open directorships (director)
        'closed_director_directorships_count',
        # integer Number of closed directorships (director)
        'secretary_directorships_count',
        # integer Number of secretary directorships
        'open_secretary_directorships_count',
        # integer Number of open secretary directorships
        'closed_secretary_directorships_count',
        # integer Number of closed secretary directorships
        'retired_secretary_directorships_count',
        # integer Number of retired decretary directorships
        'forename',
        # string Forename
        'surname',
        # string Surname
        'date_of_birth',
        # dateTime Date of Birth
        'directorships_url',
        # string Link to directorships
        'companies_url',
        # string Link to companies
        'director_url',
        # string Link to director profile
        # undocumented:
        'middle_name',
        'title',
        'postal_title',
        'nationality',
        'nation_code',
    ]

    related_resources = {
        'companies': 'pro.company.Company',
        'directorships': 'pro.company.DirectorShip',
    }




from .. import SearchResource


class DirectorSearchResult(SearchResource):
    attribute_names = [
        'name',
        'locale',
        'date_of_birth',
        'director_url',
        'directorships_url',
        'companies_url',
    ]

    result_obj = {
        # this import path is incorrect.
        'director': 'duedil.resources.pro.director.Director'
    }
    term_filters = [
        "name",
        "gender",
        "title",
        "nationality",
        "secretarial",
        "corporate",
        "disqualified",
    ]
    range_filters = [
        "age",
        "data_of_birth",
        "gross_profit",
        "gross_profit_delta_percentage",
        "turnover",
        "turnover_delta_percentage",
        "cost_of_sales",
        "cost_of_sales_delta_percentage",
        "depreciation",
        "depreciation_delta_percentage",
        "taxation",
        "cash",
        "cash_delta_percentage",
        "net_worth",
        "net_worth_delta_percentage",
        "total_assets",
        "total_assets_delta_percentage",
        "current_assets",
        "current_assets_delta_percentage",
        "net_assets",
        "net_assets_delta_percentage",
        "total_liabilities",
        "total_liabilities_delta_percentage",
    ]




from .. import SearchResource


class CompanySearchResult(SearchResource):
    attribute_names = [
        'registration_number'
        'name'
        'type'
        'status'
        'address'
        'office_type'
        'country'
        'date_of_latest_accounts'
        'online_reports'
        'available_report_types'
        'available_languages'
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from ... import (ProResource, RelatedResourceMixin)


class Company(RelatedResourceMixin, ProResource):

    path = 'companies'

    attribute_names = [
        # this is filled by __init__ and must match this value 'id',
        # integer The registered company number (ID) of the company
        'last_update',
        # dateTime Date last updated
        'name',
        # string The registered company name
        'description',
        # string A description of the company filed with the registrar
        'status',
        # string The status of the company
        'incorporation_date',
        # dateTime The date the company was incorporated
        'latest_annual_return_date',
        # dateTime Date of most recent annual return
        'latest_accounts_date',
        # dateTime Date of most recent filed accounts
        'company_type',
        # string The company type
        'accounts_type',
        # string Type of accounts
        'sic_code',
        # integer Standard Industry Classification (SIC) code
        'previous_company_names_url',
        # string Link to previous company names
        'shareholdings_url',
        # string Link to shareholders information
        'accounts_account_status',
        # integer Accounts status
        'accounts_accounts_format',
        # integer Accounts format
        'accounts_assets_current',
        # integer Current assets
        'accounts_assets_intangible',
        # integer Intangible assets
        'accounts_assets_net',
        # integer Net assets
        'accounts_assets_other_current',
        # integer Other current assets
        'accounts_assets_tangible',
        # integer Tangible assets
        'accounts_url',
        # string Link to company accounts
        'accounts_assets_total_current',
        # integer Total current assets
        'accounts_assets_total_fix',
        # integer Total fixed assets
        'accounts_audit_fees',
        # integer Audit fees
        'accounts_bank_overdraft',
        # integer Bank overdraft
        'accounts_bank_overdraft_lt_loans',
        # integer Bank overdraft & long term loans
        'accounts_capital_employed',
        # integer Capital employed
        'accounts_cash',
        # integer Cash
        'accounts_consolidated',
        # boolean Accounts consolidated (Y/N)
        'accounts_cost_of_sales',
        # integer Cost of sales
        'accounts_currency',
        # string Accounts currency
        'accounts_date',
        # dateTime Accounts date
        'accounts_depreciation',
        # integer Depreciation
        'accounts_directors_emoluments',
        # integer Directors' emoluments
        'accounts_dividends_payable',
        # integer Dividends payable
        'accounts_gross_profit',
        # integer Gross profit
        'accounts_increase_in_cash',
        # integer Increase in cash
        'accounts_interest_payments',
        # integer Interest payments
        'accounts_liabilities_current',
        # integer Current liabilities
        'accounts_liabilities_lt',
        # integer Long term liabilities
        'accounts_liabilities_misc_current',
        # integer Miscellaneous current liabilities
        'accounts_liabilities_total',
        # integer Total liabilities
        'accounts_lt_loans',
        # integer Long term loans
        'accounts_months',
        # integer Months included in accounts
        'accounts_net_cashflow_before_financing',
        # integer Net cashflow before financing
        'accounts_net_cashflow_from_financing',
        # integer Net cashflow from financing
        'accounts_net_worth',
        # integer Net worth
        'accounts_no_of_employees',
        # integer Number of employees
        'accounts_operating_profits',
        # integer Operating profits
        'accounts_operations_net_cashflow',
        # integer Net cashflow
        'accounts_paid_up_equity',
        # integer Paid-up equity
        'accounts_pandl_account_reserve',
        # integer Account reserve
        'accounts_pre_tax_profit',
        # integer Pre-tax profit
        'accounts_profit_after_tax',
        # integer Profit after tax
        'accounts_retained_profit',
        # integer Retained profit
        'accounts_shareholder_funds',
        # integer Shareholder funds
        'accounts_short_term_loans',
        # integer Short term loans
        'accounts_stock',
        # integer Stock
        'accounts_sundry_reserves',
        # integer Sundry reserves
        'accounts_taxation',
        # integer Taxation
        'accounts_trade_creditors',
        # integer Trade creditors
        'accounts_turnover',
        # integer Turnover
        'accounts_wages',
        # integer Wages
        'accounts_working_capital',
        # integer Working capital
        'directors_url',
        # string Link to company directors
        'directorships_url',
        # string Link to directorships
        'directorships_open',
        # integer Number of open directorships
        'directorships_open_secretary',
        # integer Number of current directorships with Company Secretary status
        'directorships_open_director',
        # integer Number of current directorships with Director status
        'directorships_retired',
        # integer Number of retired directorships
        'directorships_retired_secretary',
        # integer Of which secretaries
        'directorships_retired_director',
        # integer Of which directors
        'subsidiaries_url',
        # string Link to company subsidiaries
        'documents_url',
        # string Link to original company documents
        'accounts_filing_date',
        # dateTime Accounts filing date
        'ftse_a',
        # integer FTSE listing category
        'mortgage_partial_outstanding_count',
        # integer Number of partially outstanding mortgages
        'mortgage_partial_property_satisfied_count',
        # integer Number of partially satified mortgages
        'mortgage_partial_property_count',
        # integer Number of partial mortgages
        'mortgages_url',
        # string Link to mortgages
        'mortgages_outstanding_count',
        # integer Number of outstanding mortgages
        'mortgages_satisfied_count',
        # integer Number of satisfied mortgages
        'reg_address1',
        # string Registered address street
        'reg_address2',
        # string Registered address town
        'reg_address3',
        # string Registered address county
        'reg_address4',
        # string Registered address country
        'reg_address_postcode',
        # string Registered address postcode
        'reg_area_code',
        # string Registered address area code
        'reg_phone',
        # string Registered phone number
        'reg_tps',
        # boolean Telephone Preference Service (TPS) notification (Y/N)
        'reg_web',
        # string Registered web address
        'sic2007code',
        'sic2007description',
        # integer 2007 Standard Industry Classification (SIC) code
        'trading_address1',
        # string Trading address street
        'trading_address2',
        # string Trading address town
        'trading_address3',
        # string Trading address county
        'trading_address_postcode',
        # string Trading address postcode

        'charity_number',
        'liquidation_status',
        'directorships_closed_director',
        'sic_description',
        'sic_codes_count',
        'trading_address4',
        'directorships_closed',
        'credit_rating_latest_description',
        'accounts_trade_debtors',
        'directorships_closed_secretary',
        'accounts_accountants',
        'accounts_auditors',
        'accounts_contingent_liability',
        'accounts_exports',
        'accounts_qualification_code',
        'accounts_revaluation_reserve',
        'accounts_solicitors',
        'bank_accounts_url',
        'next_annual_return_date',
        'preference_shareholdings_count',
        'preference_shares_issued',
        'reg_address_town',
        'reg_address_towncode',
        'reg_care_of',
        'reg_email',
        'trading_phone',
        'trading_phone_std',
        'company_url',
        # 'turnover',
        # 'turnover_delta_percentage',
    ]

    related_resources = {
        'service-addresses': 'pro.company.ServiceAddress',
        'registered-address': 'pro.company.RegisteredAddress',
        'parent': 'pro.company.Company',
        'directors': 'pro.company.Director',
        'directorships': 'pro.company.DirectorShip',
        'accounts': 'pro.company.Account',
        'previous-company-names': 'pro.company.PreviousCompanyName',
        'industries': 'pro.company.Industry',
        'shareholders': 'pro.company.Shareholder',
        'bank-accounts': 'pro.company.BankAccount',
        'mortgages': 'pro.company.Mortgage',
        'subsidiaries': 'pro.company.Company',
        'keywords': 'pro.company.Keywords',
    }



# -*- coding: utf-8 -*-
#
#  DuedilApiClient v3 Pro
#  @copyright 2014 Christian Ledermann
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
#

from __future__ import unicode_literals


from .search.lite import CompanySearchResult as LiteCompanySearchResult, LiteSearchResourceList
from .search.pro import CompanySearchResult as ProCompanySearchResult, DirectorSearchResult, ProSearchResourceList
from .search.international import CompanySearchResult as InternationalCompanySearchResult, InternationalSearchResourceList

from .cache import configure_cache, dp_region as cache_region

import os
import json

import requests
from requests.exceptions import HTTPError

from retrying import retry

try:  # pragma: no cover
    long
except NameError:  # pragma: no cover
    # Python 3
    long = int

try:  # pragma: no cover
    unicode
except NameError:  # pragma: no cover
    # Python 3
    basestring = unicode = str

API_URLS = {
    'pro': 'http://duedil.io/v3',
    'lite': 'http://api.duedil.com/open',
    'international': 'http://api.duedil.com/international',
}
API_KEY = os.environ.get('DUEDIL_API_KEY', None)

class APILimitException(Exception):
    pass

class APIMonthlyLimitException(APILimitException):
    pass

def retry_throttling(exception):
    if isinstance(exception, HTTPError):
        if exception.response.status_code == 403:
            if exception.response.reason == "Forbidden - Over rate limit":
                if 'Developer Over Qps' in exception.response.text:
                    return True
                # elif 'Developer Over Rate' in exception.response.text:
                #     raise APIMonthlyLimitException('Monthly Limit reached for Duedil calls')
    return False


class Client(object):
    cache = None
    base_url = None

    def __init__(self, api_key=None, sandbox=False):
        'Initialise the Client with which API to connect to and what cache to use'
        self.set_api(api_key, sandbox)

    def set_api(self, api_key=None, sandbox=False):

        if not (api_key or API_KEY):
            raise ValueError("Please provide a valid Duedil API key")
        self.api_key = api_key or API_KEY

        try:
            self.base_url = API_URLS.get(self.api_type, 'lite')
        except AttributeError:
            raise ValueError('Duedil API type must be "{}"'.format('", "'.join(API_URLS.keys())))

        # Are we in a sandbox?
        self.sandbox = sandbox
        if self.sandbox:
            self.base_url = self.base_url + '/sandbox'

    def get(self, endpoint, data=None):
        return self._get(endpoint, data)

    def pre_request_hook(self, endpoint, data):
        '''This is so that custom code can be run before an api call e.g. metric collection
        This is a 'read only' method in that you cannot affect what will be sent to duedil'''
        pass

    def post_request_hook(self, response):
        '''This is so that custom code can be run after an api call e.g. metric collection
        IMPORTANT: no validation has been done on the request at this point and it is 'read only'
        you cannot affect further processing of the response'''
        pass

# - _get should probably be split a bit more to allow full urls to be called
    @cache_region.cache_on_arguments()
    @retry(retry_on_exception=retry_throttling, wait_exponential_multiplier=1000, wait_exponential_max=10000)
    def _get(self, endpoint, data=None):
        'this should become the private interface to all reequests to the api'

        result = None
        data = data or {}

        if self.api_type in ["pro", "lite"]:
            data_format = 'json'
            resp_format = '.{}'.format(data_format)
        else:
            resp_format = ''

        url = "{base_url}/{endpoint}{format}"
        prepared_url = url.format(base_url=self.base_url,
                                  endpoint=endpoint,
                                  format=resp_format)
        self.pre_request_hook(prepared_url, data)

        if not result:
            params = data.copy()
            params['api_key'] = self.api_key
            response = requests.get(prepared_url, params=params)
            self.post_request_hook(response)
            try:
                if not response.raise_for_status():
                    result = response.json()
            except HTTPError:
                if response.status_code == 404:
                    result = {}
                elif 'Developer Over Rate' in response.text:
                    raise APIMonthlyLimitException('Monthly Limit reached for Duedil calls')
                else:
                    raise

        return result

    def _search(self, endpoint, result_klass, *args, **kwargs):
        query_params = self._build_search_string(*args, **kwargs)
        results = self._get(endpoint, data=query_params)
        # return [result_klass(self, **r) for r in results.get('response',{}).get('data', {})]
        return self.search_list_class(results, result_klass, self)

    def _build_search_string(self, *args, **kwargs):
        data = {}
        try:
            data['q'] = kwargs['query']
        except KeyError:
            raise ValueError('query key must be present as a kwarg')
        return data

    def search(self, query):
        raise NotImplementedError

    def __str__(self):
        return 'Duedil Client type:{}'.format(self.api_type)

class LiteClient(Client):
    api_type = 'lite'
    search_list_class = LiteSearchResourceList

    def search(self, query):
        #  this will need to be alter in all likely hood to do some validation
        return self._search('search', LiteCompanySearchResult, query=query)


class ProClient(Client):
    api_type = 'pro'
    search_list_class = ProSearchResourceList

    def _build_search_string(self, term_filters, range_filters,
                             order_by=None, limit=None, offset=None,
                             **kwargs):
        data = {}
        for arg, value in kwargs.items():
            if arg in term_filters:
                # this must be  a string
                try:
                    assert(isinstance(value, basestring))
                except AssertionError:
                    raise TypeError('%s must be string type' % arg)
            elif arg in range_filters:
                # array of two numbers
                try:
                    assert(isinstance(value, (list, tuple)))
                except AssertionError:
                    raise TypeError('%s must be an array' % arg)
                try:
                    assert(len(value) == 2)
                except AssertionError:
                    raise ValueError('Argument %s can only be an array of length 2' % arg)
                for v in value:
                    try:
                        assert(isinstance(v, (int, long, float)))
                    except AssertionError:
                        raise TypeError('Value of %s must be numeric' % arg)
            else:
                raise TypeError('%s does not match %s' % (arg, ', '.join(term_filters+range_filters)))
        data['filters'] = json.dumps(kwargs)
        if order_by:
            try:
                assert(isinstance(order_by, dict))
            except:
                raise TypeError('order_by must be dictionary')
            try:
                assert('field' in order_by)
            except AssertionError:
                raise ValueError("'field' must be a key in the order_by dictionary")
            try:
                assert(order_by['field'] in term_filters + range_filters)
            except AssertionError:
                raise TypeError("order_by['field'] must be one of %s" % (', '.join(term_filters+range_filters)))
            if order_by.get('direction'):
                try:
                    assert(order_by['direction'] in ['asc', 'desc'])
                except AssertionError:
                    raise ValueError('The direction must either be "asc" or "desc"')
            data['orderBy'] = json.dumps(order_by)
        if limit:
            try:
                assert(isinstance(limit, int))
            except AssertionError:
                raise TypeError('limit must be an integer')
            data['limit'] = limit
        if offset:
            try:
                assert(isinstance(offset, int))
            except AssertionError:
                raise TypeError('offset must be an integer')
            data['offset'] = offset
        return data

    def search_company(self, order_by=None, limit=None, offset=None, **kwargs):
        '''
        Conduct advanced searches across all companies registered in
        UK & Ireland.
        Apply any combination of 44 different filters

        The parameter filters supports two different types of queries:
            * the "range" type (ie, a numeric range) and
            * the "terms" type (for example, an individual company name).

        For the range filter, you have to pass an array;
        for the terms filter, you just pass a string.

        The range type is used when you want to limit the results to a
        particular range of results.

        You can order the results based on the ranges using the
        parameter orderBy.
        '''
        return self._search('companies',
                            ProCompanySearchResult,
                            ProCompanySearchResult.term_filters,
                            ProCompanySearchResult.range_filters,
                            order_by=order_by,
                            limit=limit,
                            offset=offset,
                            **kwargs)

    def search_director(self, order_by=None, limit=None, offset=None, **kwargs):
        '''
        This "Director search endpoint" is similar to the
        "Company search endpoint", though with some different ranges and
        terms.

        Searching by financial range will return directors who have a
        directorship at a company fulfilling that range.

        NB: The location filter is not available for director search.
        '''
        return self._search('directors',
                            DirectorSearchResult,
                            DirectorSearchResult.term_filters,
                            DirectorSearchResult.range_filters,
                            order_by=order_by,
                            limit=limit,
                            offset=offset,
                            **kwargs)

    def search(self, order_by=None, limit=None, offset=None, **kwargs):
        return self.search_company(order_by, limit, offset, **kwargs) + \
               self.search_director(order_by, limit, offset, **kwargs)


class InternationalClient(Client):
    api_type = 'international'
    search_list_class = InternationalSearchResourceList

    def search(self, country_code, query):
        endpoint = '{}/search'.format(country_code)
        return self._search(endpoint, InternationalCompanySearchResult, query=query)

    def get(self, country_code, endpoint, data):
        endpoint = '{}/{}'.format(country_code, endpoint)
        return self._get(endpoint, data)

    # this should be a Resource and not here...
    def report(self, country_code, id):
        return self.get(country_code, 'report/{}'.format(id), {})



import os
import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as test_command


def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as f:
        return f.read()


class PyTest(test_command):

    def finalize_options(self):
        test_command.finalize_options(self)
        if sys.version_info.major < 3:
            self.test_args = []
            self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


version = '1.0.1'

setup(name='duedil',
      version=version,
      description="Duedil API client",
      long_description=(read('README.rst') + '\n\n' +
                        read('docs', 'HISTORY.rst') + '\n\n' +
                        read('docs', 'TODO.rst')),
      classifiers=['Intended Audience :: Developers',
                   'License :: OSI Approved :: Apache Software License',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.3',
                   'Programming Language :: Python :: 3.4',
                   'Development Status :: 3 - Alpha', ],
      # Get strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='duedil, api',
      author='Christian Ledermann',
      author_email='christian.ledermann@gmail.com',
      url='http://duedilv3.readthedocs.org/en/latest/',
      license='Apache License 2.0',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          'requests>=2,<3',
          'six',
          'retrying',
          'dogpile.cache',
      ],
      tests_require=['pytest', 'requests_mock'],
      cmdclass = {'test': PyTest},
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from __future__ import unicode_literals

from ... import ProResource


class Mortgage(ProResource):
    attribute_names = [
        'id',
        # string Mortgage ID
        'last_update',
        # dateTime Date last updated
        'number',
        # integer Number of charge
        'date_created',
        # dateTime Date created
        'type',
        # string Type of charge
        'date_registered',
        # dateTime Date registered
        'persons_entitled',
        # string Persons entitled
        'date_satisfied',
        # dateTime Date satisfied
        'status',
        # string Status
        'details',
        # string Details
        'amount_secured',
        # string Amount secured
    ]



from __future__ import unicode_literals

from .accounts import Account
from .accounts.financial import AccountDetailsFinancial
from .accounts.gaap import AccountDetailsGAAP
from .accounts.ifrs import AccountDetailsIFRS
from .accounts.insurance import AccountDetailsInsurance
from .accounts.statutory import AccountDetailsStatutory
from .bank_account import BankAccount
from .company import Company
from .director import Director
from .directorship import Directorship
from .document import Document
from .secondary_industries import Industry
from .mortgage import Mortgage
from .previous_company_name import PreviousCompanyName
from .registered_address import RegisteredAddress
from .service_address import ServiceAddress
from .shareholder import Shareholder
from .company_keywords import Keywords

__all__ = ['AccountDetailsFinancial', 'AccountDetailsGAAP',
           'AccountDetailsIFRS', 'AccountDetailsInsurance',
           'AccountDetailsStatutory', 'BankAccount',
           'Company', 'Director', 'Directorship', 'Document',
           'Industry', 'Mortgage', 'Keywords',
           'PreviousCompanyName', 'RegisteredAddress',
           'ServiceAddress', 'Shareholder']
